# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

import unittest

from traits.api import HasTraits, Supports
from traits.interface_checker import check_implements
from traitsui.api import ITreeNode

from envisage.ui.single_project.api import Project
from envisage.ui.single_project.view.project_view import EmptyProject


class MyProject(Project):
    pass


class MyView(HasTraits):
    tree_node = Supports(ITreeNode)


class TestProjectView(unittest.TestCase):
    def test_empty_project_adapts_to_i_tree_node(self):
        empty_project = EmptyProject()

        view = MyView()
        view.tree_node = empty_project

        adapted = view.tree_node
        check_implements(type(adapted), ITreeNode)

    def test_project_adapts_to_i_tree_node(self):
        my_project = MyProject()

        view = MyView()
        view.tree_node = my_project

        adapted = view.tree_node
        check_implements(type(adapted), ITreeNode)



# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

""" An IPython kernel plugin. """

import logging
import warnings

# Enthought library imports.
from envisage.api import (
    bind_extension_point, ExtensionPoint, Plugin, ServiceOffer)
from traits.api import Bool, Instance, List


# Extension point IDs.
SERVICE_OFFERS = 'envisage.service_offers'
IPYTHON_NAMESPACE = 'ipython_plugin.namespace'

# Protocol for the contributed service offer.
IPYTHON_KERNEL_PROTOCOL = 'envisage.plugins.ipython_kernel.internal_ipkernel.InternalIPKernel'  # noqa: E501

logger = logging.getLogger(__name__)


class IPythonKernelPlugin(Plugin):
    """ An IPython kernel plugin. """

    #: The plugin unique identifier.
    id = 'envisage.plugins.ipython_kernel'

    #: The plugin name (suitable for displaying to the user).
    name = 'IPython embedded kernel plugin'

    #: Extension point for objects contributed to the IPython kernel namespace.
    kernel_namespace = ExtensionPoint(
        List, id=IPYTHON_NAMESPACE, desc="""

        Variables to add to the IPython kernel namespace.
        This is a list of tuples (name, value).

        """
    )

    #: Service offers contributed by this plugin.
    service_offers = List(contributes_to=SERVICE_OFFERS)

    #: Whether to initialize the kernel when the service is created.
    #: The default is ``False```, for backwards compatibility. It will change
    #: to ``True`` in a future version of Envisage. External users wanting
    #: to use the future behaviour now should pass ``init_ipkernel=True``
    #: when creating the plugin.
    init_ipkernel = Bool(False)

    def stop(self):
        """ Stop the plugin. """
        self._destroy_kernel()

    # Private traits and methods

    #: The InternalIPKernel instance provided by the service.
    _kernel = Instance(IPYTHON_KERNEL_PROTOCOL)

    def _create_kernel(self):
        from .internal_ipkernel import InternalIPKernel

        # This shouldn't happen with a normal lifecycle, but add a warning
        # just in case.
        if self._kernel is not None:
            warnings.warn(
                "A kernel already exists. "
                "No new kernel will be created.",
                RuntimeWarning,
            )
            return

        logger.debug("Creating the embedded IPython kernel")
        kernel = self._kernel = InternalIPKernel()
        bind_extension_point(kernel, 'initial_namespace',
                             IPYTHON_NAMESPACE, self.application)
        if self.init_ipkernel:
            kernel.init_ipkernel()
        else:
            warnings.warn(
                (
                    "In the future, the IPython kernel will be initialized "
                    "automatically at creation time. To enable this "
                    "future behaviour now, create the plugin using "
                    "IPythonKernelPlugin(init_ipkernel=True)"
                ),
                DeprecationWarning,
            )

        return kernel

    def _destroy_kernel(self):
        """
        Destroy any existing kernel.
        """
        if self._kernel is None:
            return

        logger.debug("Shutting down the embedded IPython kernel")
        self._kernel.shutdown()
        self._kernel = None

    def _service_offers_default(self):
        ipython_kernel_service_offer = ServiceOffer(
            protocol=IPYTHON_KERNEL_PROTOCOL,
            factory=self._create_kernel,
        )
        return [ipython_kernel_service_offer]



# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

import contextlib
import os
try:
    # Python 3: mock available in std. lib.
    from unittest import mock
except ImportError:
    # Python 2: use 3rd party mock library
    import mock
import shutil
import tempfile
import unittest
import warnings

from traits.api import List

from envisage._compat import STRING_BASE_CLASS
from envisage.api import Application, Plugin
from envisage.core_plugin import CorePlugin
from envisage.tests.ets_config_patcher import ETSConfigPatcher

# Skip these tests unless ipykernel is available.
try:
    import ipykernel  # noqa: F401
except ImportError:
    ipykernel_available = False
else:
    ipykernel_available = True

if ipykernel_available:
    from envisage.plugins.ipython_kernel.internal_ipkernel import (
        InternalIPKernel)
    from envisage.plugins.ipython_kernel.ipython_kernel_plugin import (
        IPYTHON_KERNEL_PROTOCOL, IPYTHON_NAMESPACE, IPythonKernelPlugin)


@unittest.skipUnless(ipykernel_available,
                     "skipping tests that require the ipykernel package")
class TestIPythonKernelPlugin(unittest.TestCase):
    def setUp(self):
        ets_config_patcher = ETSConfigPatcher()
        ets_config_patcher.start()
        self.addCleanup(ets_config_patcher.stop)

        # Make sure that IPython-related files are written to a temporary
        # directory instead of the home directory.
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)

        self._old_ipythondir = os.environ.get("IPYTHONDIR")
        os.environ["IPYTHONDIR"] = tmpdir

    def tearDown(self):
        # Restore previous state of the IPYTHONDIR environment variable.
        old_ipythondir = self._old_ipythondir
        if old_ipythondir is None:
            del os.environ["IPYTHONDIR"]
        else:
            os.environ["IPYTHONDIR"] = old_ipythondir

    def test_import_from_api(self):
        # Regression test for enthought/envisage#108
        from envisage.plugins.ipython_kernel.api import IPYTHON_KERNEL_PROTOCOL
        self.assertIsInstance(IPYTHON_KERNEL_PROTOCOL, STRING_BASE_CLASS)

    def test_kernel_service(self):
        # See that we can get the IPython kernel service when the plugin is
        # there.
        with self.running_app() as app:
            kernel = app.get_service(IPYTHON_KERNEL_PROTOCOL)
            self.assertIsInstance(kernel, InternalIPKernel)
            self.assertIsNotNone(kernel.ipkernel)

        # After application stop, the InternalIPKernel object should
        # also have been shut down.
        self.assertIsNone(kernel.ipkernel)

    def test_kernel_namespace_extension_point(self):
        class NamespacePlugin(Plugin):
            kernel_namespace = List(contributes_to=IPYTHON_NAMESPACE)

            def _kernel_namespace_default(self):
                return [('y', 'hi')]

        plugins = [
            IPythonKernelPlugin(init_ipkernel=True),
            NamespacePlugin(),
        ]

        with self.running_app(plugins=plugins) as app:
            kernel = app.get_service(IPYTHON_KERNEL_PROTOCOL)
            self.assertIn('y', kernel.namespace)
            self.assertEqual(kernel.namespace['y'], 'hi')

    def test_get_service_twice(self):
        with self.running_app() as app:
            kernel1 = app.get_service(IPYTHON_KERNEL_PROTOCOL)
            kernel2 = app.get_service(IPYTHON_KERNEL_PROTOCOL)
            self.assertIs(kernel1, kernel2)

    def test_service_not_used(self):
        # If the service isn't used, no kernel should be created.
        from envisage.plugins.ipython_kernel import internal_ipkernel

        kernel_instances = []

        class TrackingInternalIPKernel(internal_ipkernel.InternalIPKernel):
            def __init__(self, *args, **kwargs):
                super(TrackingInternalIPKernel, self).__init__(*args, **kwargs)
                kernel_instances.append(self)

        patcher = mock.patch.object(
            internal_ipkernel,
            "InternalIPKernel",
            TrackingInternalIPKernel,
        )

        with patcher:
            kernel_plugin = IPythonKernelPlugin(init_ipkernel=True)
            with self.running_app(plugins=[kernel_plugin]):
                pass

        self.assertEqual(kernel_instances, [])

    def test_service_used(self):
        # This is a complement to the test_service_not_used test. It's mostly
        # here as a double check on the somewhat messy test machinery used in
        # test_service_not_used: if the assumptions (e.g., on the location that
        # InternalIPKernel is imported from) in test_service_not_used break,
        # then this test will likely break too.

        from envisage.plugins.ipython_kernel import internal_ipkernel

        kernel_instances = []

        class TrackingInternalIPKernel(internal_ipkernel.InternalIPKernel):
            def __init__(self, *args, **kwargs):
                super(TrackingInternalIPKernel, self).__init__(*args, **kwargs)
                kernel_instances.append(self)

        patcher = mock.patch.object(
            internal_ipkernel,
            "InternalIPKernel",
            TrackingInternalIPKernel,
        )

        with patcher:
            kernel_plugin = IPythonKernelPlugin(init_ipkernel=True)
            with self.running_app(plugins=[kernel_plugin]) as app:
                app.get_service(IPYTHON_KERNEL_PROTOCOL)

        self.assertEqual(len(kernel_instances), 1)

    def test_no_init(self):
        # Testing deprecated behaviour where the kernel is not initialized.
        plugins = [IPythonKernelPlugin()]

        with self.running_app(plugins=plugins) as app:
            with warnings.catch_warnings(record=True) as warn_msgs:
                warnings.simplefilter("always", category=DeprecationWarning)
                app.get_service(IPYTHON_KERNEL_PROTOCOL)

        matching_messages = [
            msg for msg in warn_msgs
            if isinstance(msg.message, DeprecationWarning)
            if "kernel will be initialized" in str(msg.message)
        ]
        self.assertEqual(len(matching_messages), 1)

    @contextlib.contextmanager
    def running_app(self, plugins=None):
        """
        Returns a context manager that provides a running application.

        Parameters
        ----------
        plugins : list of Plugin, optional
            Plugins to use in the application, other than the CorePlugin
            (which is always included). If not given, an IPythonKernelPlugin
            is instantiated and used.
        """
        if plugins is None:
            plugins = [IPythonKernelPlugin(init_ipkernel=True)]

        app = Application(plugins=[CorePlugin()] + plugins, id='test')
        app.start()
        try:
            yield app
        finally:
            app.stop()



# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
""" Plugin finder for the current package. """


def get_plugins():
    """ Get the plugins from this package. """

    from .orange_plugin import OrangePlugin

    return [OrangePlugin()]

#### EOF #######################################################################



# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

"""
Version information for this Envisage distribution.

This file is autogenerated by the Envisage setup.py script.
"""
from __future__ import unicode_literals

#: The full version of the package, including a development suffix
#: for unreleased versions of the package.
version = "4.9.1"

#: The Git revision from which this release was made.
git_revision = "05bbaad61309d863bfd5723e8dd794dd1f655574"



# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
from .new_project_action import NewProjectAction
from .open_project_action import OpenProjectAction
from .save_project_action import SaveProjectAction
from .save_as_project_action import SaveAsProjectAction
from .close_project_action import CloseProjectAction
from .switch_to_action import SwitchToAction
